module Expri
  class Dumper
    def initialize(options={})
      @options = options
    end

    def run
      str = ""
      str << header
      metrics = Metric.list
      metrics.each do |metric|
        metric = Metric.new(metric, {}, @options)
        metric.get
        str << dump_metric(metric)
      end
      str << footer
    end

    private

    NUMERICAL_ATTRIBUTES = %w{divisor frequency precentile window}.
      map { |x| x.to_sym }.freeze

    def dump_metric(metric)
      attributes = metric.attributes_without_defaults.dup

      type           = attributes.delete(:type)
      predicate      = attributes.delete(:predicate)

      # at bare minimum, window is required, so attributes size should always
      # be > 0 even after type and predicate are removed
      attributes_str = attributes.to_a.
        sort_by { |a| a[0] }.
        map { |a| "#{a[0]}: #{NUMERICAL_ATTRIBUTES.include?(a[0]) ? a[1] : %{"#{a[1]}"}}" }.
        join(",\n    ")

      # predicate is important, but optional
      str =  %{  metric "#{metric.name}",\n}
      str << %{    type: :#{type},\n}
      str << %{    predicate: #{predicate},\n} if predicate
      str << %{    #{attributes_str}\n\n}
    end

    def footer
      "end"
    end

    def header
      "Expri.metrics(destroy_extras: false) do\n\n"
    end
  end
end
