module Expri
  class Metric
    ATTRIBUTE_DEFAULTS = { :frequency => 3 }.freeze
    VALID_ATTRIBUTES = %w{divisor frequency key percentile predicate predicate2
      type window value}.map { |x| x.to_sym }.freeze

    attr_accessor :attributes, :name, :options

    def self.list(options={})
      metric = Metric.new(nil, {}, {})
      MultiJson.decode(metric.api.get(path: "/metrics", expects: 200).body)
    end

    def initialize(name, attributes={}, options={})
      @name           = name
      @options        = options
      self.attributes = attributes
    end

    def ==(other)
      attributes_without_defaults == other.attributes_without_defaults
    end

    def api
      @api ||= Excon.new(api_url)
    end

    def attributes=(attributes)
      # symbolize keys
      attributes.keys.each do |key|
        attributes[(key.to_sym rescue key) || key] = attributes.delete(key)
      end

      attributes.delete_if { |k, v| !VALID_ATTRIBUTES.include?(k) }
      attributes.delete_if { |k, v| v == nil }

      # manual normalization
      attributes[:type] = attributes[:type].to_sym if attributes[:type]

      @attributes = attributes
    end

    # only the attributes that are not at their default value
    def attributes_without_defaults
      attributes = @attributes.dup
      attributes.delete_if { |k, v| ATTRIBUTE_DEFAULTS[k] == v }
    end

    def destroy
      puts "destroy name=#{name}" if @options[:verbose]
      api.delete(path: "/metrics/#{name}", body: MultiJson.encode(@attributes),
        expects: 200)
    end

    def get
      puts "show name=#{name}" if @options[:verbose]
      self.attributes =
        MultiJson.decode(api.get(path: "/metrics/#{name}", expects: 200).body)
    end

    def post
      puts "create name=#{name}" if @options[:verbose]
      # exprd's put is like most api's post
      api.put(path: "/metrics/#{name}", body: MultiJson.encode(@attributes),
        expects: 200)
    rescue Excon::Errors::BadRequest => e
      message = MultiJson.decode(e.response.body)["message"]
      raise(message)
    end

    def put
      begin
        existing = Metric.new(name, {}, @options)
        existing.get
        if self != existing
          existing.destroy
          post
        end
      rescue Excon::Errors::NotFound
        post
      end
    end

    private

    def api_url
      @options[:exprd_api_url] || ENV["EXPRD_API_URL"] ||
        raise("need EXPRD_API")
    end
  end
end
